import java.io.*;
import java.util.*;
import java.net.*;

public class HTTPSession implements Runnable
{
   private WebServerFrame frame;
   private Socket clientSocket;

   String linesep, filesep;

   // Stream of socket
   DataInputStream httpRequest;
   DataOutputStream htmlResponse;

   // Constructor
   public HTTPSession(Socket clientSocket, WebServerFrame frame) throws IOException
   {
      this.clientSocket = clientSocket;
      this.frame = frame;

      httpRequest  = new DataInputStream(clientSocket.getInputStream());
      htmlResponse = new DataOutputStream(clientSocket.getOutputStream());

      // Line Separator
      linesep = System.getProperty("line.separator");
      // File Separator
      filesep = System.getProperty("file.separator");
   }

   public void run()
   {
      String request = "";
      byte[] buffer = new byte[1024];
      int index = 0;
      try
      {
	 while(true)
	 {
	    httpRequest.readFully(buffer, index, 1);

	    index++;
	    // Check the request if end
	    // The end of requst header should be \r\n\r\n
	    if ( buffer[index-1]==(byte)10 && index >= 4 )
	       if ( buffer[index-4]==(byte)13 && buffer[index-3]==(byte)10 && buffer[index-2]==(byte)13 )
		  break;
	 }
	 request = new String(buffer, 0, index);
	 frame.txtRequest.append(request + linesep);
	 frame.txtRequest.setCaretPosition(frame.txtRequest.getText().length());

	 StringTokenizer stRequest = new StringTokenizer(request);

	 // The HTTP request Header must have 2 more tokens
	 // First token is method
	 // Second token is request file
	 if ((stRequest.countTokens()>= 2) && stRequest.nextToken().equalsIgnoreCase("GET"))
	 {
	    // Check the request file
	    if ((request = stRequest.nextToken()).startsWith("/"))
	    {
	       request = request.substring(1);
	    }
	    if (request.endsWith("/") || request.equals(""))
	    {
	       request = request + frame.txtDoc.getText();
	    }
	    request = frame.txtDir.getText() + filesep + request ;
	    sendHTMLResponse(request);
	 }
	 else
	 {
	    // Send 400 response the client
	    request = frame.txtDir.getText() + filesep + "error" + filesep + "400.html" ;
	    sendHTMLResponse(request);
	 }
      }
      catch (SecurityException se)
      {
	 try
	 {
	    // Send 403 response the client
	    request = frame.txtDir.getText() + filesep + "error" + filesep + "403.html" ;
	    sendHTMLResponse(request);
	    se.printStackTrace();
	 }
	 catch (IOException ioexc)
	 {
	    ioexc.printStackTrace();
	 }
      }
      catch (FileNotFoundException fnfe)
      {
	 try
	 {
	    // Send 404 response the client
	    request = frame.txtDir.getText() + filesep + "error" + filesep + "404.html" ;
	    sendHTMLResponse(request);
	    fnfe.printStackTrace();
	 }
	 catch (IOException ioex)
	 {
	    ioex.printStackTrace();
	 }
      }
      catch (IOException ioe)
      {
	 ioe.printStackTrace();
      }
      catch (Exception e)
      {
	 e.printStackTrace();
      }
      finally
      {
	 // Log the exception and close the socket
	 try
	 {
	    frame.txtLog.append("Connection close from " + clientSocket.getInetAddress().getHostAddress() + linesep);
	    frame.txtLog.setCaretPosition(frame.txtLog.getText().length());

	    clientSocket.close();
	 }
	 catch (Exception e) {}
      }
   }

   // Send the response to client
   void sendHTMLResponse(String request) throws IOException
   {
      File requestFile = new File(request);

      DataInputStream requestStream = new DataInputStream(new FileInputStream(requestFile));
      int len = (int) requestFile.length();
      byte[] buffer = new byte[len];

      requestStream.readFully(buffer);

      // Header of response
      String responseHeader = "HTTP/1.0 200 OK\r\n" +
	 "Server: Java WebServer 1.0\r\n" +
	 "Content-Length: " + buffer.length +"\r\n" +
	 "Content-Type: " + getContentType(request) + "\r\n\r\n" ;

      // Write response header and request file
      htmlResponse.writeBytes(responseHeader);
      htmlResponse.write(buffer);
      htmlResponse.flush();
      requestStream.close();

      frame.txtResponse.append(request + linesep + responseHeader);
      frame.txtResponse.setCaretPosition(frame.txtResponse.getText().length());
   }

   // Check MIME type
   private static String getContentType(String request)
   {
      if (request.endsWith("html"))
	 return "text/html";
      else if (request.endsWith("htm"))
	 return "text/html";
      else if (request.endsWith("txt"))
	 return "text/plain";
      else if (request.endsWith("gif"))
	 return "image/gif";
      else if (request.endsWith("jpg"))
	 return "image/jpeg";
      else if (request.endsWith("jpeg"))
	 return "image/jpeg";
      else if (request.endsWith("pdf"))
	 return "application/pdf";
      else if (request.endsWith("pdf"))
	 return "application/pdf";
      else if (request.endsWith("doc"))
	 return "application/msword";
      else if (request.endsWith("xls"))
	 return "application/vnd.ms-excel";
      else if (request.endsWith("ppt"))
	 return "application/vnd.ms-powerpoint";
      else
	 return "text/plain";
   }
}
